@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.64 2010/08/17 02:22:16 kitagawa Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.65.2.4 2011/08/30 11:31:14 kitagawa Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2010	PgPool Global Development Group
+ * Copyright (c) 2003-2011	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -1303,6 +1303,7 @@ void pool_free_startup_packet(StartupPacket *sp)
 			free(sp->user);
 		free(sp);
 	}
+	sp = NULL;
 }
 
 /*
@@ -1465,6 +1466,40 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	return cp;
 }
 
+/*
+ * Discard connection and memory allocated by
+ * make_persistent_db_connection().
+ */
+void discard_persistent_db_connection(POOL_CONNECTION_POOL_SLOT *cp)
+{
+	int len;
+
+	if(cp == NULL)
+		return;
+
+	pool_write(cp->con, "X", 1);
+	len = htonl(4);
+	pool_write(cp->con, &len, sizeof(len));
+
+	/*
+	 * XXX we cannot call pool_flush() here since backend may already
+	 * close the socket and pool_flush() automatically invokes fail
+	 * over handler. This could happen in copy command (remember the
+	 * famous "lost synchronization with server, resetting
+	 * connection" message)
+	 */
+	pool_set_nonblock(cp->con->fd);
+	pool_flush_it(cp->con);
+	pool_unset_nonblock(cp->con->fd);
+
+	pool_close(cp->con);
+	free(cp->sp->startup_packet);
+	free(cp->sp->database);
+	free(cp->sp->user);
+	free(cp->sp);
+	free(cp);
+}
+
 /*
  * do authentication for cp
  */
