@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.85.2.9 2011/05/25 17:13:16 gleu Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.85.2.11 2011/08/11 00:29:38 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
@@ -1281,7 +1281,8 @@ void send_failback_request(int node_id)
 	Req_info->kind = NODE_UP_REQUEST;
 	Req_info->node_id[0] = node_id;
 
-	if (node_id < 0 || node_id >= MAX_NUM_BACKENDS || VALID_BACKEND(node_id))
+    if (node_id < 0 || node_id >= MAX_NUM_BACKENDS || 
+		(RAW_MODE && BACKEND_INFO(node_id).backend_status != CON_DOWN && VALID_BACKEND(node_id)))
 	{
 		pool_error("send_failback_request: node %d is alive.", node_id);
 		return;
@@ -1459,7 +1460,8 @@ static void failover(void)
 	if (Req_info->kind == NODE_UP_REQUEST)
 	{
 		if (node_id >= MAX_NUM_BACKENDS ||
-			(Req_info->kind == NODE_UP_REQUEST && VALID_BACKEND(node_id)) ||
+			(Req_info->kind == NODE_UP_REQUEST && !(RAW_MODE &&
+            BACKEND_INFO(node_id).backend_status == CON_DOWN) && VALID_BACKEND(node_id)) ||
 			(Req_info->kind == NODE_DOWN_REQUEST && !VALID_BACKEND(node_id)))
 		{
 			pool_semaphore_unlock(REQUEST_INFO_SEM);
