@@ -435,7 +435,7 @@ typedef struct Node
  *	Fortunately, this macro isn't recursive so we just define
  *	a global variable for this purpose.
  */
-extern PGDLLIMPORT Node *newNodeMacroHolder;
+extern Node *newNodeMacroHolder;
 
 #define newNode(size, tag) \
 ( \
