@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_connection_pool.c,v 1.30.2.1 2011/01/07 01:24:17 kitagawa Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_connection_pool.c,v 1.30.2.2 2011/06/15 01:29:44 kitagawa Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2010	PgPool Global Development Group
+ * Copyright (c) 2003-2011	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -467,7 +467,7 @@ int connect_unix_domain_socket_by_port(int port, char *socket_dir, bool retry)
 	fd = socket(AF_UNIX, SOCK_STREAM, 0);
 	if (fd == -1)
 	{
-		pool_error("connect_unix_domain_socket_by_port: setsockopt() failed: %s", strerror(errno));
+		pool_error("connect_unix_domain_socket_by_port: socket() failed: %s", strerror(errno));
 		return -1;
 	}
 
