@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_process_reporting.c,v 1.14 2010/08/14 06:42:07 gleu Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_process_reporting.c,v 1.15.2.3 2011/05/23 09:34:17 gleu Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
@@ -31,7 +31,7 @@
 #include <netinet/in.h>
 
 /* some length definitions */
-#define POOLCONFIG_MAXNAMELEN 32
+#define POOLCONFIG_MAXNAMELEN 64
 #define POOLCONFIG_MAXVALLEN 512
 #define POOLCONFIG_MAXDESCLEN 64
 #define POOLCONFIG_MAXIDENTLEN 63
@@ -200,6 +200,11 @@ void config_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 	strncpy(status[i].desc, "stop replication mode on fatal error", POOLCONFIG_MAXDESCLEN);
 	i++;
 
+	strncpy(status[i].name, "failover_if_affected_tuples_mismatch", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->failover_if_affected_tuples_mismatch);
+	strncpy(status[i].desc, "failover if affected tuples are mismatch", POOLCONFIG_MAXDESCLEN);
+	i++;
+
 	strncpy(status[i].name, "replicate_select", POOLCONFIG_MAXNAMELEN);
 	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->replicate_select);
 	strncpy(status[i].desc, "non 0 if SELECT statement is replicated", POOLCONFIG_MAXDESCLEN);
@@ -298,13 +303,13 @@ void config_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 	i++;
 
 	strncpy(status[i].name, "failback_command", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->failover_command);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->failback_command);
 	strncpy(status[i].desc, "failback command", POOLCONFIG_MAXDESCLEN);
 	i++;
 
 	strncpy(status[i].name, "fail_over_on_backend_error", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->insert_lock);
-	strncpy(status[i].desc, "fail_over_on_backend_error", POOLCONFIG_MAXDESCLEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->fail_over_on_backend_error);
+	strncpy(status[i].desc, "fail over on backend error", POOLCONFIG_MAXDESCLEN);
 	i++;
 
 	strncpy(status[i].name, "insert_lock", POOLCONFIG_MAXNAMELEN);
@@ -377,11 +382,6 @@ void config_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 	strncpy(status[i].desc, "online recovery user", POOLCONFIG_MAXDESCLEN);
 	i++;
 
-	strncpy(status[i].name, "recovery_password", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->recovery_password);
-	strncpy(status[i].desc, "online recovery password", POOLCONFIG_MAXDESCLEN);
-	i++;
-
 	strncpy(status[i].name, "recovery_1st_stage_command", POOLCONFIG_MAXNAMELEN);
 	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->recovery_1st_stage_command);
 	strncpy(status[i].desc, "execute a command in first stage.", POOLCONFIG_MAXDESCLEN);
@@ -467,11 +467,6 @@ void config_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 	strncpy(status[i].desc, "user name to access system DB", POOLCONFIG_MAXDESCLEN);
 	i++;
 
-	strncpy(status[i].name, "system_db_password", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->system_db_password);
-	strncpy(status[i].desc, "password to access system DB", POOLCONFIG_MAXDESCLEN);
-	i++;
-
 	for (j = 0; j < NUM_BACKENDS; j++)
 	{
 		if (BACKEND_INFO(j).backend_port == 0)
@@ -492,7 +487,7 @@ void config_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 		snprintf(status[i].desc, POOLCONFIG_MAXDESCLEN, "weight of backend #%d", j);
 		i++;
 
-		snprintf(status[i].name, POOLCONFIG_MAXNAMELEN, "backend status%d", j);
+		snprintf(status[i].name, POOLCONFIG_MAXNAMELEN, "backend_status%d", j);
 		snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", BACKEND_INFO(j).backend_status);
 		snprintf(status[i].desc, POOLCONFIG_MAXDESCLEN, "status of backend #%d", j);
 		i++;
