@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_proto_modules.c,v 1.89.2.9 2011/05/26 05:04:10 kitagawa Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_proto_modules.c,v 1.89.2.14 2011/10/31 02:09:49 kitagawa Exp $
  * 
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -113,6 +113,7 @@ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 
 	POOL_SESSION_CONTEXT *session_context;
 	POOL_QUERY_CONTEXT *query_context;
+	POOL_MEMORY_POOL *old_context;
 
 	/* Get session context */
 	session_context = pool_get_session_context();
@@ -146,22 +147,40 @@ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 		return POOL_END;
 	}
 
+	/* switch memory context */
+	old_context = pool_memory;
+	pool_memory = query_context->memory_context;
+
 	/* parse SQL string */
 	parse_tree_list = raw_parser(contents);
 
 	if (parse_tree_list == NIL)
 	{
-		/*
-		 * Unable to parse the query. Probably syntax error or the
-		 * query is too new and our parser cannot understand. Treat as
-		 * if it were an DELETE command. Note that the DELETE command
-		 * does not execute, instead the original query will be sent
-		 * to backends, which may or may not cause an actual syntax errors.
-		 * The command will be sent to all backends in replication mode
-		 * or master/primary in master/slave mode.
-		 */
-		pool_log("SimpleQuery: Unable to parse the query: %s", contents);
-		parse_tree_list = raw_parser(POOL_DUMMY_QUERY);
+		/* is the query empty? */
+		if (*contents == '\0' || *contents == ';')
+		{
+			/*
+			 * JBoss sends empty queries for checking connections.
+			 * We replace the empty query with SELECT command not
+			 * to affect load balance.
+			 * [Pgpool-general] Confused about JDBC and load balancing
+			 */
+			parse_tree_list = raw_parser(POOL_DUMMY_READ_QUERY);
+		}
+		else
+		{
+			/*
+			 * Unable to parse the query. Probably syntax error or the
+			 * query is too new and our parser cannot understand. Treat as
+			 * if it were an DELETE command. Note that the DELETE command
+			 * does not execute, instead the original query will be sent
+			 * to backends, which may or may not cause an actual syntax errors.
+			 * The command will be sent to all backends in replication mode
+			 * or master/primary in master/slave mode.
+			 */
+			pool_log("SimpleQuery: Unable to parse the query: %s", contents);
+			parse_tree_list = raw_parser(POOL_DUMMY_WRITE_QUERY);
+		}
 	}
 
 	if (parse_tree_list != NIL)
@@ -254,7 +273,6 @@ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 					 sp->user, sp->database, remote_ps_data);
 			set_ps_display(psbuf, false);
 
-			free_parser();
 			pool_query_context_destroy(query_context);
 			pool_set_skip_reading_from_backends();
 			return POOL_CONTINUE;
@@ -319,7 +337,7 @@ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 					status = insert_lock(frontend, backend, contents, (InsertStmt *)node, lock_kind);
 					if (status != POOL_CONTINUE)
 					{
-						free_parser();
+						pool_query_context_destroy(query_context);
 						return status;
 					}
 				}
@@ -327,7 +345,7 @@ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 		}
 		else if (REPLICATION && contents == NULL && start_internal_transaction(frontend, backend, node))
 		{
-			free_parser();
+			pool_query_context_destroy(query_context);
 			return POOL_ERROR;
 		}
 	}
@@ -379,7 +397,11 @@ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 		   	{
 				POOL_SENT_MESSAGE *msg = NULL;
 
-				if (IsA(node, ExecuteStmt))
+				if (IsA(node, PrepareStmt))
+				{
+					msg = session_context->uncompleted_message;
+				}
+				else if (IsA(node, ExecuteStmt))
 				{
 					msg = pool_get_sent_message('Q', ((ExecuteStmt *)node)->name);
 					if (!msg)
@@ -408,16 +430,14 @@ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 			{
 				/* Send query to all DB nodes at once */
 				status = pool_send_and_wait(query_context, string, len, 0, 0, "");
-				/*
-				free_parser();
-				*/
+				/* free_parser(); */
 				return status;
 			}
 
 			/* Send the query to master node */
 			if (pool_send_and_wait(query_context, string, len, 1, MASTER_NODE_ID, "") != POOL_CONTINUE)
 			{
-				free_parser();
+				pool_query_context_destroy(query_context);
 				return POOL_END;
 			}
 		}
@@ -427,7 +447,7 @@ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 		 */
 		if (pool_send_and_wait(query_context, string, len, -1, MASTER_NODE_ID, "") != POOL_CONTINUE)
 		{
-			free_parser();
+			pool_query_context_destroy(query_context);
 			return POOL_END;
 		}
 
@@ -436,25 +456,25 @@ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 		{
 			if (pool_send_and_wait(query_context, string, len, 1, MASTER_NODE_ID, "") != POOL_CONTINUE)
 			{
-/*
-				free_parser();
-*/
+				pool_query_context_destroy(query_context);
 				return POOL_END;
 			}
 		}
-		free_parser();
+		/* free_parser(); */
 	}
 	else
 	{
 		if (pool_send_and_wait(query_context, string, len, 1, MASTER_NODE_ID, "") != POOL_CONTINUE)
 		{
-			free_parser();
+			pool_query_context_destroy(query_context);
 			return POOL_END;
 		}
-/*
-		free_parser();
-*/
+		/* free_parser(); */
 	}
+
+	/* switch memory context */
+	pool_memory = old_context;
+
 	return POOL_CONTINUE;
 }
 
@@ -626,17 +646,31 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 
 	if (parse_tree_list == NIL)
 	{
-		/*
-		 * Unable to parse the query. Probably syntax error or the
-		 * query is too new and our parser cannot understand. Treat as
-		 * if it were an DELETE command. Note that the DELETE command
-		 * does not execute, instead the original query will be sent
-		 * to backends, which may or may not cause an actual syntax errors.
-		 * The command will be sent to all backends in replication mode
-		 * or master/primary in master/slave mode.
-		 */
-		pool_log("Parse: Unable to parse the query: %s", contents);
-		parse_tree_list = raw_parser(POOL_DUMMY_QUERY);
+		/* is the query empty? */
+		if (*stmt == '\0' || *stmt == ';')
+		{
+			/*
+			 * JBoss sends empty queries for checking connections.
+			 * We replace the empty query with SELECT command not
+			 * to affect load balance.
+			 * [Pgpool-general] Confused about JDBC and load balancing
+			 */
+			parse_tree_list = raw_parser(POOL_DUMMY_READ_QUERY);
+		}
+		else
+		{
+			/*
+			 * Unable to parse the query. Probably syntax error or the
+			 * query is too new and our parser cannot understand. Treat as
+			 * if it were an DELETE command. Note that the DELETE command
+			 * does not execute, instead the original query will be sent
+			 * to backends, which may or may not cause an actual syntax errors.
+			 * The command will be sent to all backends in replication mode
+			 * or master/primary in master/slave mode.
+			 */
+			pool_log("Parse: Unable to parse the query: %s", stmt);
+			parse_tree_list = raw_parser(POOL_DUMMY_WRITE_QUERY);
+		}
 	}
 
 	if (parse_tree_list != NIL)
@@ -728,13 +762,13 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 			kind = pool_read_kind(backend);
 			if (kind != 'Z')
 			{
-				/* free_parser(); */
+				pool_query_context_destroy(query_context);
 				return POOL_END;
 			}
 
 			if (ReadyForQuery(frontend, backend, 0) != POOL_CONTINUE)
 			{
-				/* free_parser(); */
+				pool_query_context_destroy(query_context);
 				return POOL_END;
 			}
 
@@ -755,7 +789,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 			status = insert_lock(frontend, backend, stmt, (InsertStmt *)query_context->parse_tree, insert_stmt_with_lock);
 			if (status != POOL_CONTINUE)
 			{
-				/* free_parser(); */
+				pool_query_context_destroy(query_context);
 				return status;
 			}
 		}
@@ -775,7 +809,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 		pool_debug("Parse: waiting for master completing the query");
 		if (pool_send_and_wait(query_context, contents, len, 1, MASTER_NODE_ID, "P") != POOL_CONTINUE)
 		{
-			/* free_parser(); */
+			pool_query_context_destroy(query_context);
 			return POOL_END;
 		}
 
@@ -788,7 +822,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 		deadlock_detected = detect_deadlock_error(MASTER(backend), MAJOR(backend));
 		if (deadlock_detected < 0)
 		{
-			/* free_parser(); */
+			pool_query_context_destroy(query_context);
 			return POOL_END;
 		}
 		else
@@ -809,7 +843,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 								   strlen(POOL_ERROR_QUERY)+1, -1,
 								   MASTER_NODE_ID, "") != POOL_CONTINUE)
 			{
-				/* free_parser(); */
+				pool_query_context_destroy(query_context);
 				return POOL_END;
 			}
 		}
@@ -817,7 +851,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 		{
 			if (pool_send_and_wait(query_context, contents, len, -1, MASTER_NODE_ID, "P") != POOL_CONTINUE)
 			{
-				/* free_parser(); */
+				pool_query_context_destroy(query_context);
 				return POOL_END;
 			}
 		}
@@ -826,7 +860,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 	{
 		if (pool_send_and_wait(query_context, contents, len, 1, MASTER_NODE_ID, "P") != POOL_CONTINUE)
 		{
-			/* free_parser(); */
+			pool_query_context_destroy(query_context);
 			return POOL_END;
 		}
 	}
@@ -1317,7 +1351,7 @@ POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 			 * If 2PC commands, automatically close transaction on standbys since
 			 * 2PC commands close transaction on primary.
 			 */
-			else if (is_2pc_transaction_query(node, query))
+			else if (is_2pc_transaction_query(node))
 			{
 				for (i=0;i<NUM_BACKENDS;i++)
 				{
