@@ -1,7 +1,7 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool-II/pool_query_context.c,v 1.32.2.7 2011/05/26 05:04:09 kitagawa Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_query_context.c,v 1.32.2.11 2011/08/18 02:28:08 kitagawa Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -114,7 +114,7 @@ void pool_set_node_to_be_sent(POOL_QUERY_CONTEXT *query_context, int node_id)
 
 	if (node_id < 0 || node_id > MAX_NUM_BACKENDS)
 	{
-		pool_error("pool_set_node_to_be_sent: invalid ndoe id:%d", node_id);
+		pool_error("pool_set_node_to_be_sent: invalid node id:%d", node_id);
 		return;
 	}
 
@@ -136,7 +136,7 @@ void pool_unset_node_to_be_sent(POOL_QUERY_CONTEXT *query_context, int node_id)
 
 	if (node_id < 0 || node_id > MAX_NUM_BACKENDS)
 	{
-		pool_error("pool_unset_node_to_be_sent: invalid ndoe id:%d", node_id);
+		pool_error("pool_unset_node_to_be_sent: invalid node id:%d", node_id);
 		return;
 	}
 
@@ -224,7 +224,7 @@ bool pool_is_node_to_be_sent(POOL_QUERY_CONTEXT *query_context, int node_id)
 
 	if (node_id < 0 || node_id > MAX_NUM_BACKENDS)
 	{
-		pool_error("pool_is_node_to_be_sent: invalid ndoe id:%d", node_id);
+		pool_error("pool_is_node_to_be_sent: invalid node id:%d", node_id);
 		return false;
 	}
 
@@ -414,6 +414,15 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 						pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
 					}
 
+					/*
+					 * If unlogged table is used in the SELECT,
+					 * we prefer to send to the primary.
+					 */
+					else if (pool_has_unlogged_table(node))
+					{
+						pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
+					}
+
 					else
 					{
 						pool_set_node_to_be_sent(query_context,
@@ -625,7 +634,7 @@ POOL_STATUS pool_send_and_wait(POOL_QUERY_CONTEXT *query_context, char *string,
 			continue;
 		}
 
-		if (wait_for_query_response(frontend, CONNECTION(backend, i), string, MAJOR(backend)) != POOL_CONTINUE)
+		if (wait_for_query_response(frontend, CONNECTION(backend, i), MAJOR(backend)) != POOL_CONTINUE)
 		{
 			/* Cancel current transaction */
 			CancelPacket cancel_packet;
@@ -781,14 +790,9 @@ static POOL_DEST send_to_where(Node *node, char *query)
 		 */
 		if (IsA(node, SelectStmt))
 		{
-			/* SELECT INTO? */
-			if (((SelectStmt *)node)->intoClause)
+			/* SELECT INTO or SELECT FOR SHARE or UPDATE ? */
+			if (pool_has_insertinto_or_locking_clause(node))
 				return POOL_PRIMARY;
-
-			/* SELECT FOR SHARE or UPDATE */
-			else if (((SelectStmt *)node)->lockingClause)
-				return POOL_PRIMARY;
-
 			/*
 			 * SELECT nextval(), setval()
 			 * XXX: We do not search in subquery.
@@ -856,7 +860,7 @@ static POOL_DEST send_to_where(Node *node, char *query)
 			/*
 			 * 2PC commands
 			 */
-			else if (is_2pc_transaction_query(node, query))
+			else if (is_2pc_transaction_query(node))
 				return POOL_PRIMARY;
 			else
 				/* COMMIT etc. */
@@ -1115,7 +1119,7 @@ bool is_set_transaction_serializable(Node *node, char *query)
 /*
  * Return true if the query is 2PC transaction query.
  */
-bool is_2pc_transaction_query(Node *node, char *query)
+bool is_2pc_transaction_query(Node *node)
 {
 	if (((TransactionStmt *)node)->kind == TRANS_STMT_PREPARE ||
 		((TransactionStmt *)node)->kind == TRANS_STMT_COMMIT_PREPARED ||
